//
// Multi-Timeframe Rainbow Trend Follower.mq4 
// Written By Forex Factory Member: Robert Alan Byer
//                                  byer@mail.ourservers.net
//
// Based On Original Work By Forex Factory Member: Paul Murfin
//
// This creates a seperate indicator window to display the trends of the
// GMMA "rainbow" trend , MCACD trend, Laguerre trend and EMA crossover trend signals 
// and gives an overall trading signal suggestion.
//
// The user can specify if they want an trading alert to be given as well as an e-mail alert.
// (The setting of "Trade_Alerts" and "EMail_Alerts" triggers this)
//
// The user can also specify if they want to see the indicators for the M1, M5, M30, H1
// H4, D1, W1 and MN timeframes instead of the current chart timeframe.
// (The setting of "Multitimeframe" triggers this)
//
// The user can also specify if they want to display if the London, New York, Toyko and
// Sydney markets are open or closed.
// (The setting of "Display_MarketStatus" triggers this)
//
// The user can also specify if they would like to see the bid/ask spread for the chart pair.
// (The setting of "Display_BidAskSpread" triggers this)
//
// This is based on the rules of the "Rainbow Trend Follower" system outlined at
// http://www.bustathief.com/earn-money-with-forex-trend-follower/
// http://www.forexfactory.com/showthread.php?t=40325
//
#property copyright "Robert Alan Byer"
#property link      "byer@mail.ourservers.net"
#define VERSION "v1.1"

//
// Create the indicator in a seperate window.
//
#property indicator_separate_window
//
// Mimimum indicator value will be 0.
//
#property indicator_minimum 0
//
// Maximum indicator value will be 1.
//
#property indicator_maximum 1

//
// Define the parameters that the user can change.
//
extern bool Trade_Alerts = true;            // Define if we want trade alerts.
extern bool EMail_Alerts = false;           // Define if we want e-mail alerts.
extern bool Multitimeframe =true;           // Define if we want multitime frame indicator.
extern bool Display_MarketStatus = true;    // Define if we want to display the market open/close indicators.
extern bool Display_BidAskSpread = true;    // Define if we want to display the bid/ask spread of the current pair.
extern int GMT_Offset = 0;                  // Define the server offset from GMT.

//
// Define chart parameters.
//
int X_Scale=25;                       // Define the horizontal interval which the chart "squares" are created.
int Y_Scale=20;                       // Define the verfical interval which the chart "squares" are created.
int X_Offset=35;                      // Define the horizontal indentation of all the chart "squares".
int Y_Offset=30;                      // Define the verfical indentation of all the chart "squares".
string Symbol_Font = "Wingdings";     // Define the chart symbol font.
int Symbol_Font_Size=15;              // Define the chart symbol font size.
int Description_Font_Size=8;          // Define the chart column & row description font size.
string Description_Font = "Tacoma";   // Define the chart column & row description font.
int Description_Color = Black;        // Define the chart objects color.
int Long_Symbol = 221;                // Define the symbol for a long signal.
int Short_Symbol = 222;               // Define the symbol for a short signal.
int Flat_Symbol = 163;                // Define the symbol for a flat signal.
int Long_Symbol_Color = Green;        // Define the symbol color for a long signal.
int Short_Symbol_Color = Red;         // Define the symbol color for a short signal.
int Flat_Symbol_Color = Black;        // Define the symbol color for a flat signal.

//
// Define our chart timeframes.
//
int Chart_Timeframe[9] = {PERIOD_M1, PERIOD_M5, PERIOD_M15};
//
// Define our chart time period description string.
//
string Chart_Period_String[9] = {"M1", "M5", "M15"};
//
// Define our "long" text description.
//
string Long_String = "Long";
//
// Define our "short" text description.
//
string Short_String = "Short";
//
// Define our "flat" text description.
//
string Flat_String = "Flat";

//
// Define our GMMA chat row item headers.
//
string GMMA_Row_String[6] = {"Yellow GMMA", "Orange GMMA", "Blue GMMA", "Green GMMA", "Red GMMA", "Investor Trend"};
//
// Define our GMMA trend array.
//
// Yello GMMA M1, M5, M15, M30, H1, H4, D1, W1, MN trend.
// Orange GMMA M1, M5, M15, M30, H1, H4, D1, W1, MN trend.
// Blue GMMA M1, M5, M15, M30, H1, H4, D1, W1, MN trend.
// Green GMMA M1, M5, M15, M30, H1, H4, D1, W1, MN trend.
// Red GMMA M1, M5, M15, M30, H1, H4, D1, W1, MN trend.
//
string GMMA_Trend[6][9];
//
// Define our GMMA Max array.
//
// Yello GMMA Max M1, M5, M15, M30, H1, H4, D1, W1, MN.
// Orange GMMA Max M1, M5, M15, M30, H1, H4, D1, W1, MN.
// Blue GMMA Max M1, M5, M15, M30, H1, H4, D1, W1, MN.
// Green GMMA Max M1, M5, M15, M30, H1, H4, D1, W1, MN.
// Red GMMA Max M1, M5, M15, M30, H1, H4, D1, W1, MN.
//
double GMMA_Max[5][9];
//
// Define our GMMA Min array.
//
// Yello GMMA Min M1, M5, M15, M30, H1, H4, D1, W1, MN.
// Orange GMMA Min M1, M5, M15, M30, H1, H4, D1, W1, MN.
// Blue GMMA Min M1, M5, M15, M30, H1, H4, D1, W1, MN.
// Green GMMA Min M1, M5, M15, M30, H1, H4, D1, W1, MN.
// Red GMMA Min M1, M5, M15, M30, H1, H4, D1, W1, MN.
//
double GMMA_Min[5][9];

//
// Define our trend chart row item headers.
//
string Trend_Row_String[4] = {"EMA Crossover", "MACD", "Laguerre", "Overall Guppy", "Trade Suggestion"};
//
// Define our trend array.
//
// EMA Crossover.
// MACD.
// Laguerre.
// Overall GMMA.
// Trade Suggestion.
//
string Trend_Array[5][9];
//
// Define our MACD value array.
//
double MACD_Value[9];
//
// Define our Laqguerre value array.
//
double Laguerre_Value[9];

//
// Define the the time of the last chart bar for the various chart timeframes.
//
datetime Last_BarTime[9];

//+------------------------------------------------------------------+
//| Initialize our custom indicator.                                 |
//+------------------------------------------------------------------+
int init()
{
   int timeframe_loop;                 // Loop variable to step through the
                                       // Chart_Timeframe array.
                                       
   //
   // Check to see if we are going to display multitimeframe data.
   //
   if (Multitimeframe)
   {
      //
      // Create a loop to count through our Chart_Timeframe array.
      //
      for (timeframe_loop=0; timeframe_loop<ArraySize(Chart_Timeframe); timeframe_loop++)
      {
         //
         // Initialize the Last_BarTime array by checking for new chart bars for the chart period.
         //
         Check_NewBar(timeframe_loop, Chart_Timeframe[timeframe_loop]);
         //
         // Compute the indicators for the time period.
         //
         Compute_Indicators(timeframe_loop, Chart_Timeframe[timeframe_loop]);
         //
         // Display the GMMA and trend indicators for the time period.
         //
         Draw_Indicators(timeframe_loop);
      }   
   }
   else
   {
      //
      // Initialize the Last_BarTime for this chart time period.
      //
      Check_NewBar(0, 0);
      //
      // Compute the indicators for the current chart time period.
      //
      Compute_Indicators(0, Period());
      //
      // Display the GMMA and trend indicators for the current chart time period.
      //
      Draw_Indicators(0);
   }
   //
   // Check to see if we are to display the market open/close status.
   //
   if (Display_MarketStatus)
   {
      //
      // Draw the market indicators.
      //
      Market_Indicators();
   }
   //
   // Check to see if we are to display the bid/ask spread.
   //
   if (Display_BidAskSpread)
   {
      //
      // Draw the bid/ask spread indicator.
      //
      BidAskSpread_Indicator();
   }
    
   return(0);
}
   
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
{
   //
   // Delete All Objects When We Delete The Indicator.
   //
   ObjectsDeleteAll();
 
   return(0);
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{

   int timeframe_loop;                 // Loop variable to step through the
                                       // Chart_Timeframe array.
                                    
   //
   // Check to see if we are going to display multitimeframe data.
   //
   if (Multitimeframe)
   {
      //
      // Create a loop to count through our Chart_Timeframe array.
      //
      for (timeframe_loop=0; timeframe_loop<ArraySize(Chart_Timeframe); timeframe_loop++)
      {
         //
         // Check to see if we have a new chart bar for the chart period.
         //
         if (Check_NewBar(timeframe_loop, Chart_Timeframe[timeframe_loop]))
         {
            //
            // Compute the indicators for the time period.
            //
            Compute_Indicators(timeframe_loop, Chart_Timeframe[timeframe_loop]);
            //
            // Display the GMMA and trend indicators for the time period.
            //
            Draw_Indicators(timeframe_loop);
            //
            // Check to see if the user wants trade alerts.
            //
            if (Trade_Alerts)
            {
               //
               // The user wants trade alerts so check to see if we have a "long" trade suggestion.
               //
               if (Trend_Array[4][timeframe_loop] == Long_String)
               {
                  Alert("Rainbow Trend Follower Alert: " + Symbol() + " " + Chart_Period_String[timeframe_loop] + " - GO LONG, BUY!!!");
               }
               //
               // The user wants trade alerts so check to see if we have a "short" trade suggestion.
               //
               if (Trend_Array[4][timeframe_loop] == Short_String)
               {
                  Alert("Rainbow Trend Follower Alert: " + Symbol() + " " + Chart_Period_String[timeframe_loop] + " - GO SHORT, SELL!!!");
               }
            }
            //
            // Check to see if the user wants e-mail trade alerts and if our trade suggestion is not "flat".
            //
            if (EMail_Alerts && Trend_Array[4][timeframe_loop] != Flat_String)
            {
               //
               // The trade suggestionis not "flat" so send an e-mail notice.
               //
               Send_Email(timeframe_loop, Chart_Timeframe[timeframe_loop]);
            }
         }   
      }
   }
   else
   {
      //
      // Check to see if we have a new chart bar for the chart period.
      //
      if (Check_NewBar(0, 0))
      {
         //
         // Compute the indicators for the current chart time period.
         //
         Compute_Indicators(0, Period());
         //
         // Display the GMMA and trend indicators for the current chart time period.
         //
         Draw_Indicators(0);
         //
         // Check to see if the user wants trade alerts.
         //
         if (Trade_Alerts)
         {
            //
            // The user wants trade alerts so check to see if we have a "long" trade suggestion.
            //
            if (Trend_Array[4][0] == Long_String)
            {
               Alert("Rainbow Trend Follower Alert: " + Symbol() + " " + Chart_Period_Name(Period()) + " - GO LONG, BUY!!!");
            }
            //
            // The user wants trade alerts so check to see if we have a "short" trade suggestion.
            //
            if (Trend_Array[4][0] == Short_String)
            {
               Alert("Rainbow Trend Follower Alert: " + Symbol() + " " + Chart_Period_Name(Period()) + " - GO SHORT, SELL!!!");
            }
         }
         //
         // Check to see if the user wants e-mail trade alerts and if our trade suggestion is not "flat".
         //
         if (EMail_Alerts && Trend_Array[4][0] != Flat_String)
         {
            //
            // The trade suggestion is not "flat" so send an e-mail notice.
            //
            Send_Email(0, Period());
         }
      }
   }
   //
   // Check to see if we are to display the market open/close status.
   //
   if (Display_MarketStatus)
   {
      //
      // Draw the market indicators.
      //
      Market_Indicators();
   }
   //
   // Check to see if we are to display the bid/ask spread.
   //
   if (Display_BidAskSpread)
   {
      //
      // Draw the bid/ask spread indicator.
      //
      BidAskSpread_Indicator();
   }
        
   return(0);
}

//+------------------------------------------------------------------+
//| Draw_Indicators.                                                 |
//| Written By: Robert Alan Byer                                     |
//|             byer@mail.ourservers.net                             |
//|                                                                  |
//|                                                                  |
//| Display the "yellow", "orange", "blue", "green" and "red"        |
//| "rainbow" GMMA trends and the EMA crossover, MACD, Laguerre and  |
//| overall guppy trends as well as an overall trade suggestion      |
//| for time periods passed to us for the "Rainbow Trend Follower"   |
//| system outlined at..                                             |
//|                                                                  |
//| http://www.bustathief.com/earn-money-with-forex-trend-follower/  |
//| http://www.forexfactory.com/showthread.php?t=40325               |
//+------------------------------------------------------------------+
void Draw_Indicators(int timeframe_index)
{

   int num_rows;                       // Our chart row counter.
   int multitimeframe_offset;          // The indicator offset for multi-timeframes.
   string GMMA_Chart_Object_Name;      // Our GMMA chart object name string.
   string Trend_Chart_Object_Name;     // Our trend chart object name string.
   int Indicator_Window_Index;         // Our indicator window index integer.   
   
   //
   // Check to see if we are displaying multitimeframes.
   //
   if (Multitimeframe)
   {
      //
      // Set the offset for multitimeframe indicators.
      //
      multitimeframe_offset = 340;
   }
   else
   {
      //
      // Set the offset for singletimeframe indicators.
      //
      multitimeframe_offset = 135;
   }   
   //
   // Get the window index for the "Rainbow Trend Follower" indicator window.
   //
   Indicator_Window_Index = WindowFind("Multi-Timeframe Rainbow Trend Follower " + VERSION);
   //
   // Check to see if the indicator window is visible.
   //
   if(Indicator_Window_Index < -1 && !WindowIsVisible(Indicator_Window_Index))
   {
      Print("The Multi-Timeframe Rainbow Trend Follower " + VERSION + " Indicator Window Was Not Found Or Is Not Visible...");
      return(0);
   }
   //
   // Create a loop for the number of elements in our GMMA_Row_String.
   //
   for(num_rows=0; num_rows<ArraySize(GMMA_Row_String); num_rows++)
   {
      //
      // Check to see if we are displaying multitimeframes.
      //
      if (Multitimeframe)
      {
         //
         // Create our GMMA row and define our chart object name string by taking the GMMA_Row_String 
         // for this chart row and adding the Chart_Period_String to the end of it.
         //
         GMMA_Chart_Object_Name = GMMA_Row_String[num_rows] + " " + Chart_Period_String[timeframe_index];
      }
      else
      {
         //
         // Create our GMMA row and define our chart object name string by taking the GMMA_Row_String 
         // for this chart row and adding the chart period to the end of it.
         //
         GMMA_Chart_Object_Name = GMMA_Row_String[num_rows] + " " + Chart_Period_Name(Period());
      }         
      ObjectCreate(GMMA_Chart_Object_Name,OBJ_LABEL,Indicator_Window_Index,0,0,0,0);
      ObjectSet(GMMA_Chart_Object_Name,OBJPROP_XDISTANCE,timeframe_index*X_Scale+X_Offset+50);
      ObjectSet(GMMA_Chart_Object_Name,OBJPROP_YDISTANCE,num_rows*Y_Scale+Y_Offset);        
      //
      // Check to see if the trend for this part of the "rainbow" for the timeframe is "long".
      //
      if(GMMA_Trend[num_rows][timeframe_index] == Long_String)
      {
         //
         // The GMMA_Trend for this "rainbow" color for the timeframe is "long" so set the "long" indicator object.
         //
         ObjectSetText(GMMA_Chart_Object_Name,CharToStr(Long_Symbol),Symbol_Font_Size,Symbol_Font,Long_Symbol_Color);
      }
      else
      {
         //
         // Check to see if the trend for this part of the "rainbow" for the timeframe is "short".
         //
         if(GMMA_Trend[num_rows][timeframe_index] == Short_String)
         {
            //
            // The GMMA_Trend for this "rainbow" color for the timeframe is "short" so set the "short" indicator object.
            //
            ObjectSetText(GMMA_Chart_Object_Name,CharToStr(Short_Symbol),Symbol_Font_Size,Symbol_Font,Short_Symbol_Color);
         }
         else
         {
            //
            // Check to see if the trend for this part of the "rainbow" for the timeframe is "flat".
            //
            if(GMMA_Trend[num_rows][timeframe_index] == Flat_String)
            {
               //
               // The GMMA_Trend for this "rainbow" color for the timeframe is "flat" so set the "flat" indicator object.
               //
               ObjectSetText(GMMA_Chart_Object_Name,CharToStr(Flat_Symbol),Symbol_Font_Size,Symbol_Font,Flat_Symbol_Color);
            }
         }
      }
      //
      // Check to see if num_rows is less than or equal to the number of elements in the Trend_Row_String array.
      //
      if (num_rows < ArraySize(Trend_Row_String))
      {
         //
         // Check to see if we are displaying multitimeframes.
         //
         if (Multitimeframe)
         {
            //
            // Create our trend row and define our chart object name string by taking the Trend_Row_String 
            // for this chart row and adding the Chart_Period_String to the end of it.
            //
            Trend_Chart_Object_Name = Trend_Row_String[num_rows] + " " + Chart_Period_String[timeframe_index];
         }
         else
         {
            //
            // Create our trend row and define our chart object name string by taking the Trend_Row_String 
            // for this chart row and adding the chart period to the end of it.
            //
            Trend_Chart_Object_Name = Trend_Row_String[num_rows] + " " + Chart_Period_Name(Period());
         }
         ObjectCreate(Trend_Chart_Object_Name,OBJ_LABEL,Indicator_Window_Index,0,0,0,0);
         ObjectSet(Trend_Chart_Object_Name,OBJPROP_XDISTANCE,timeframe_index*X_Scale+X_Offset+multitimeframe_offset+50);
         ObjectSet(Trend_Chart_Object_Name,OBJPROP_YDISTANCE,num_rows*Y_Scale+Y_Offset);        
         //
         // Check to see if the trend for the timeframe is "long".
         //
         if(Trend_Array[num_rows][timeframe_index] == Long_String)
         {
            //
            // The trend for the timeframe is "long" so set the "long" indicator object.
            //
            ObjectSetText(Trend_Chart_Object_Name,CharToStr(Long_Symbol),Symbol_Font_Size,Symbol_Font,Long_Symbol_Color);
         }
         else
         {
            //
            // Check to see if the trend for the timeframe is "short".
            //
            if(Trend_Array[num_rows][timeframe_index] == Short_String)
            {
               //
               // The trend for the timeframe is "short" so set the "short" indicator object.
               //
               ObjectSetText(Trend_Chart_Object_Name,CharToStr(Short_Symbol),Symbol_Font_Size,Symbol_Font,Short_Symbol_Color);
            }
            else
            {
               //
               // Check to see if the trend for the timeframe is "flat".
               //
               if(Trend_Array[num_rows][timeframe_index] == Flat_String)
               {
                  //
                  // The trend for the timeframe is "flat" so set the "flat" indicator object.
                  //
                  ObjectSetText(Trend_Chart_Object_Name,CharToStr(Flat_Symbol),Symbol_Font_Size,Symbol_Font,Flat_Symbol_Color);
               }
            }
         }
      } 
   }
   //
   // Check to see if we are to display multitimeframes.
   //
   if (Multitimeframe)
   {     
      //
      // Create the GMMA chart column "Time Period" labels with the data from "Chart_Period_String" array.
      //
      ObjectCreate(Chart_Period_String[timeframe_index] + " GMMA",OBJ_LABEL,Indicator_Window_Index,0,0,0,0);
      ObjectSet(Chart_Period_String[timeframe_index] + " GMMA",OBJPROP_XDISTANCE,timeframe_index*X_Scale+X_Offset+52);
      ObjectSet(Chart_Period_String[timeframe_index] + " GMMA",OBJPROP_YDISTANCE,Y_Offset-10);
      ObjectSetText(Chart_Period_String[timeframe_index] + " GMMA",Chart_Period_String[timeframe_index],Description_Font_Size,Description_Font,Description_Color);
      //
      // Create the trend chart column "Time Period" labels with the data from "Chart_Period_String" array.
      //
      ObjectCreate(Chart_Period_String[timeframe_index] + " Trend",OBJ_LABEL,Indicator_Window_Index,0,0,0,0);
      ObjectSet(Chart_Period_String[timeframe_index] + " Trend",OBJPROP_XDISTANCE,timeframe_index*X_Scale+X_Offset+multitimeframe_offset+52);
      ObjectSet(Chart_Period_String[timeframe_index] + " Trend",OBJPROP_YDISTANCE,Y_Offset-10);
      ObjectSetText(Chart_Period_String[timeframe_index] + " Trend",Chart_Period_String[timeframe_index],Description_Font_Size,Description_Font,Description_Color);
   }
   else
   {
      //
      // Create the GMMA chart column "Time Period" labels with the data from the current chart period.
      //
      ObjectCreate(Chart_Period_Name(Period()) + " GMMA",OBJ_LABEL,Indicator_Window_Index,0,0,0,0);
      ObjectSet(Chart_Period_Name(Period()) + " GMMA",OBJPROP_XDISTANCE,timeframe_index*X_Scale+X_Offset+52);
      ObjectSet(Chart_Period_Name(Period()) + " GMMA",OBJPROP_YDISTANCE,Y_Offset-10);
      ObjectSetText(Chart_Period_Name(Period()) + " GMMA",Chart_Period_Name(Period()),Description_Font_Size,Description_Font,Description_Color);
      //
      // Create the trend chart column "Time Period" labels with the data from the current chart period.
      //
      ObjectCreate(Chart_Period_Name(Period()) + " Trend",OBJ_LABEL,Indicator_Window_Index,0,0,0,0);
      ObjectSet(Chart_Period_Name(Period()) + " Trend",OBJPROP_XDISTANCE,timeframe_index*X_Scale+X_Offset+multitimeframe_offset+52);
      ObjectSet(Chart_Period_Name(Period()) + " Trend",OBJPROP_YDISTANCE,Y_Offset-10);
      ObjectSetText(Chart_Period_Name(Period()) + " Trend",Chart_Period_Name(Period()),Description_Font_Size,Description_Font,Description_Color);
   }     
   //
   // Create the GMMA and trend chart row labels with the data from the "GMMA_Row_String"  and
   // "Trend_Row_String" array.
   //
   for(num_rows=0; num_rows<ArraySize(GMMA_Row_String); num_rows++)
   {
      //
      // Create The GMMA chart row labels.
      //
      ObjectCreate(GMMA_Row_String[num_rows],OBJ_LABEL,Indicator_Window_Index,0,0,0,0);
      ObjectSet(GMMA_Row_String[num_rows],OBJPROP_XDISTANCE,X_Offset-25);
      ObjectSet(GMMA_Row_String[num_rows],OBJPROP_YDISTANCE,num_rows*Y_Scale+Y_Offset + 5);
      ObjectSetText(GMMA_Row_String[num_rows],GMMA_Row_String[num_rows],Description_Font_Size,Description_Font,Description_Color);
      //
      // Check to see if num_rows is less than or equal to the number of elements in the Trend_Row_String array.
      //
      if (num_rows < ArraySize(Trend_Row_String))
      {
         //
         // Create the trend chart row labels.
         //
         ObjectCreate(Trend_Row_String[num_rows],OBJ_LABEL,Indicator_Window_Index,0,0,0,0);
         ObjectSet(Trend_Row_String[num_rows],OBJPROP_XDISTANCE,X_Offset+(multitimeframe_offset-15)-25);
         ObjectSet(Trend_Row_String[num_rows],OBJPROP_YDISTANCE,num_rows*Y_Scale+Y_Offset + 5);
         ObjectSetText(Trend_Row_String[num_rows],Trend_Row_String[num_rows],Description_Font_Size,Description_Font,Description_Color);
      }
   }   
   return(0);
}

//+------------------------------------------------------------------+
//| Market_Indicators.                                               |
//| Written By: Robert Alan Byer                                     |
//|             byer@mail.ourservers.net                             |
//|                                                                  |
//| Indicate if the London, New York, Toyko and Syndney markets are  |
//| open or closed.                                                  |
//+------------------------------------------------------------------+
void Market_Indicators()
{
   
   int num_markets;              // Our markets counter.
   string Chart_Object_Name;     // Our chart object name string.
   int Indicator_Window_Index;   // Our indicator window index integer.   
   //
   // Define our market name array.
   //
   string Market_Array[4] = {" Frankfurt", "London", "New York", "Toyko", "Sydney"};
   //
   // Define our market opening times in terms of GMT.
   //
   // Frankfurt   07:00 GMT
   // London      08:00 GMT
   // New York    24:00 GMT
   // Toyko       23:00 GMT
   // Sydney      21:00 GMT
   //
   int Market_Open[4] = {7, 7, 12, 0, 0};
   //
   // Define our market closing times in terms of GMT.
   //
   // Frankfurt   15:00 GMT
   // London      16:00 GMT
   // New York    20:00 GMT
   // Toyko       19:00 GMT
   // Sydney      17:00 GMT
   //
   int Market_Close[5] = {16, 21, 9, 6};
   
   //
   // Get the window index for the "Rainbow Trend Follower" indicator window.
   //
   Indicator_Window_Index = WindowFind("Multi-Timeframe Rainbow Trend Follower " + VERSION);
   //
   // Check to see if the indicator window is visible.
   //
   if(Indicator_Window_Index < -1 && !WindowIsVisible(Indicator_Window_Index))
   {
      Print("The Multi-Timeframe Rainbow Trend Follower " + VERSION + " Indicator Window Was Not Found Or Is Not Visible...");
      return(0);
   }
   //
   // We have 4 columns in our trend indicator, one for each market.
   //
   for(num_markets=0; num_markets<ArraySize(Market_Array); num_markets++)
   {
      ObjectCreate(Market_Array[num_markets]+ " Market",OBJ_LABEL,Indicator_Window_Index,0,0,0,0);
      ObjectSet(Market_Array[num_markets] + " Market",OBJPROP_XDISTANCE,num_markets*(X_Scale+70)+X_Offset-20);
      ObjectSet(Market_Array[num_markets] + " Market",OBJPROP_YDISTANCE,Y_Offset+135);
      ObjectSetText(Market_Array[num_markets] + " Market",Market_Array[num_markets] + " Market",Description_Font_Size,Description_Font,Description_Color);
      //
      // Create our "market status" indicator.
      //
      ObjectCreate(Market_Array[num_markets]+ " Market Status",OBJ_LABEL,Indicator_Window_Index,0,0,0,0);
      ObjectSet(Market_Array[num_markets] + " Market Status",OBJPROP_XDISTANCE,num_markets*(X_Scale+70)+X_Offset-4);
      ObjectSet(Market_Array[num_markets] + " Market Status",OBJPROP_YDISTANCE,Y_Offset+145);
      //
      // Check to see if the market open time is less than the market close time and that the current time is greater
      // than or equal to the market open time and the current time is less than the market close.
      //
      if (Market_Open[num_markets] +GMT_Offset < Market_Close[num_markets] + GMT_Offset && (Hour() >= Market_Open[num_markets] + GMT_Offset && Hour() < Market_Close[num_markets] + GMT_Offset))
		{
		    //
		    // The market is open so display the "open" indicator.
		    //
          ObjectSetText(Market_Array[num_markets] + " Market Status","OPEN",Description_Font_Size,Description_Font,Green);		    
      }
      else
      { 
        //
        // Check to see if the market open time is greater than the market close time and that the current time
        // is greater than or equal to the market open time or the current time is less than the market closing
        // time.
        //    
   	  if (Market_Open[num_markets] + GMT_Offset > Market_Close[num_markets] + GMT_Offset && (Hour() >= Market_Open[num_markets] + GMT_Offset || Hour() < Market_Close[num_markets] + GMT_Offset))
        {
		    //
		    // The market is open so display the "open" indicator.
		    //
          ObjectSetText(Market_Array[num_markets] + " Market Status","OPEN",Description_Font_Size,Description_Font,Green);		    
        }
        else
        {
          //
          // The market is closed so display the "closed" indicator.
          //
          ObjectSetText(Market_Array[num_markets] + " Market Status","CLOSED",Description_Font_Size,Description_Font,Red);
        }
      }
   }
   return(0);
}

//+------------------------------------------------------------------+
//| BidAskSpread_Indicator.                                          |
//| Written By: Robert Alan Byer                                     |
//|             byer@mail.ourservers.net                             |
//|                                                                  |
//| Indicate the spread of the current pair.                         |
//+------------------------------------------------------------------+
void BidAskSpread_Indicator()
{

   int bidask_offset;            // The offset for the bid/ask spread indicator.
   int Indicator_Window_Index;   // Our indicator window index integer.
   
   //
   // Check to see if we are to display the market status indicator.
   //
   if (Display_MarketStatus)
   {
      //
      // We are to display the market status indicator so set the bidask_offset to 
      // take this into consideration.
      //
      bidask_offset = 460;
   }
   else
   {
      //
      // We are not going to display the market status indicator so set the bidask_offset
      // to put the indicator where the market status indicator would normally go.
      //
      bidask_offset = 0;
   }
   //
   // Get the window index for the "Rainbow Trend Follower" indicator window.
   //
   Indicator_Window_Index = WindowFind("Multi-Timeframe Rainbow Trend Follower " + VERSION);
   //
   // Check to see if the indicator window is visible.
   //
   if(Indicator_Window_Index < -1 && !WindowIsVisible(Indicator_Window_Index))
   {
      Print("The Multi-Timeframe Rainbow Trend Follower " + VERSION + " Indicator Window Was Not Found Or Is Not Visible...");
      return(0);
   }
   ObjectCreate("Spread Label",OBJ_LABEL,Indicator_Window_Index,0,0,0,0);
   ObjectSet("Spread Label",OBJPROP_XDISTANCE,(X_Scale+bidask_offset)+X_Offset-20);
   ObjectSet("Spread Label",OBJPROP_YDISTANCE,Y_Offset+135);
   ObjectSetText("Spread Label","Bid/Ask Spread",Description_Font_Size,Description_Font,Description_Color);
   //
   // Create our "market status" indicator.
   //
   ObjectCreate("Bid/Ask Spread",OBJ_LABEL,Indicator_Window_Index,0,0,0,0);
   ObjectSet("Bid/Ask Spread",OBJPROP_XDISTANCE,(X_Scale+bidask_offset)+13+X_Offset-4);
   ObjectSet("Bid/Ask Spread",OBJPROP_YDISTANCE,Y_Offset+145);
   ObjectSetText("Bid/Ask Spread",DoubleToStr(MarketInfo(Symbol(),MODE_SPREAD),0),Description_Font_Size,Description_Font,Description_Color);

   return(0);
}

//+------------------------------------------------------------------+
//| Compute_Indicators                                               |
//| Written By: Robert Alan Byer                                     |
//|             byer@mail.ourservers.net                             |
//|                                                                  |
//| Compute the indicators for the time periods passed to us for the |
//| "Rainbow Trend Follower" system outlined at...                   |
//|                                                                  |
//| http://www.bustathief.com/earn-money-with-forex-trend-follower/  |
//| http://www.forexfactory.com/showthread.php?t=40325               |
//+------------------------------------------------------------------+
void Compute_Indicators(int timeframe_index, int timeframe_period)
{

   //
   // Compute the GMMA multitimeframe trends.
   //
   Compute_GMMA_Trend(timeframe_index, timeframe_period);
   //
   // Compute the EMA crossover multitimeframe trends.
   //
   Compute_EMA_Crossover_Trend(timeframe_index, timeframe_period);
   //
   // Compute the MACD multitimeframe trends.
   //
   Compute_MACD_Trend(timeframe_index, timeframe_period);
   //
   // Compute the Laguerre multitimeframe trends.
   //
   Compute_Laguerre_Trend(timeframe_index, timeframe_period);
   //
   // Compute the overall GMMA multitimeframe trends.
   //
   Compute_Overall_GMMA_Trend(timeframe_index);
   //
   // Compute the multitimeframe trade suggestion.
   //
   Compute_Trade_Suggestion(timeframe_index);
   
   return(0);
}   

//+------------------------------------------------------------------+
//| Compute_GMMA_Trend                                               |
//| Written By: Robert Alan Byer                                     |
//|             byer@mail.ourservers.net                             |
//|                                                                  |
//| Based On Original Work By Forex Factory Member: Paul Murfin      |
//|                                                                  |
//| Compute the "yellow", "orange", "blue", "green" and "red"        |
//| GMMA values for the time periods passed to us for the            |
//| "Rainbow Trend Follower" system outlined at..                    |
//|                                                                  |
//| http://www.bustathief.com/earn-money-with-forex-trend-follower/  |
//| http://www.forexfactory.com/showthread.php?t=40325               |
//+------------------------------------------------------------------+
void Compute_GMMA_Trend(int timeframe_index, int timeframe_period)
{
                          
   int array_loop;          // Loop variable to step through the 
                            // GMMA_Array_Size array.
                            
   int period_loop;         // Loop variable to step through the 
                            // GMMA_Period array.
                            
   int GMMA_Count;          // GMMA Counter.
   double GMMA_Current;     // Current GMMA value.
   double GMMA_Previous;    // Previous GMMA value.
   double GMMAMin;          // GMMA Min.
   double GMMAMax;          // GMMA Max.

   //
   // Define our GMMA time period array size.
   //
   // Yellow GMMA period array size = 14
   // Orange GMMA period array size = 13
   // Blue GMMA period array size = 11
   // Green GMMA period array size = 12
   // Red GMMA period array size = 16
   //
   int GMMA_Array_Size[5] = {14, 13, 11, 12, 16};
   
   //
   // Define our GMMA Periods.
   // (we are padding with "0" out to 16 array elements)
   //
   // Yellow GMMA periods = 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15
   // Orange GMMA periods = 17, 19, 21, 23, 25, 27, 29, 31, 33, 35, 37, 39, 41
   // Blue GMMA periods = 44, 47, 50, 53, 56, 59, 62, 65, 68, 71, 74
   // Green GMMA periods = 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122
   // Red GMMA periods = 125, 130, 135, 140, 145, 150, 155, 160, 165, 170, 175, 180, 185, 190, 195, 200
   //
   int GMMA_Period[5][16] = {2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 0, 0, 
                             17, 19, 21, 23, 25, 27, 29, 31, 33, 35, 37, 39, 41, 0, 0, 0, 
                             44, 47, 50, 53, 56, 59, 62, 65, 68, 71, 74, 0, 0, 0, 0, 0, 
                             78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 0, 0, 0, 0, 
                             125, 130, 135, 140, 145, 150, 155, 160, 165, 170, 175, 180, 185, 190, 195, 200};
   
   //
   // Create a loop to count through our GMMA_Array_Size array.
   //
   for (array_loop=0; array_loop<ArraySize(GMMA_Array_Size); array_loop++)
   {
      //
      // Set GMMA_Count to "0".
      //
      GMMA_Count = 0;
      //
      // Set GMMA_Current to "0".
      //
      GMMA_Current = 0;
      //
      // Set GMMA_Previous to "0".
      //
      GMMA_Previous = 0;
      //
      // Set GMMAMax to "0".
      //
      GMMAMax = 0;
      //
      // Set GMMAMin to "0".
      //
      GMMAMin = 0;
      //
      // Create a loop to count through our GMMA_Period array getting the number of array elements
      // from the GMMA_Array_Size array.
      //
      for (period_loop=0; period_loop<GMMA_Array_Size[array_loop]; period_loop++)
      {
         //
         // Calculate the current moving average for the current chart symbol using the timeframe taken
         // from the timeframe_period and the period taken from the GMMA_Period array with
         // a "0" MA shift, using EMA mode on the closed price with a shift of 1.
         //
         GMMA_Current = iMA(NULL, timeframe_period, GMMA_Period[array_loop][period_loop], 0, MODE_EMA, PRICE_CLOSE, 1);
         //
         // Calculate the previous moving average for the current chart symbol using the timeframe taken
         // from the timeframe_period and the period taken from the GMMA_Period array with
         // a "0" MA shift, using EMA mode on the closed price with a shift of 2.
         //
         GMMA_Previous = iMA(NULL, timeframe_period, GMMA_Period[array_loop][period_loop], 0, MODE_EMA, PRICE_CLOSE, 2);
         //
         // Check to see if GMMA_Max array element for this timeframe equals 0 or if GMMA_Current is greater than 
         // the GMMA_Max array element for this timeframe.
         //
         if (GMMAMax == 0 || GMMA_Current > GMMAMax) 
         {
            //
            // GMMAMax equals 0 or GMMA_Current is greater than GMMAMax so make GMMAMax the GMMA_Current value.
            //
            // (If GMMAMax equals 0, we don't have a GMMAMax value yet so make GMMAMax equal the GMMA_Current 
            //  value.)
            //
            // (If GMMA_Current is greater than GMMAMax then we have a new GMMAMax value and make GMMAMax equal 
            //  the GMMA_Current value.)
            //
            GMMAMax = GMMA_Current;
         }
         //
         // Check to see if GMMAMin equals 0 or if GMMA_Current is less than GMMAMin.
         //
         if (GMMAMin == 0 || GMMA_Current < GMMAMin)
         {
            //
            // GMMAMin equals 0 or GMMA_Current is less than GMMAMin so make GMMAMin the GMMA_Current value.
            //
            // (If GMMAMin equals 0, we don't have a GMMAMin value yet so make GMMAMin equal the GMMA_Current 
            //  value.)
            //
            // (If GMMA_Current is less than GMMAMin then we have a new GMMAMin value and make GMMAMin equal 
            //  the GMMA_Current value.)
            //
            GMMAMin = GMMA_Current;
         }            
         //
         // Check to see if GMMA_Current it greater than GMMA_Previous.
         //
         if (GMMA_Current > GMMA_Previous) 
         {
            //
            // GMMA_Current is greater than GMMA_Previous so increment the GMMA_Count by 1.
            //
            // (If GMMA_Current is greater than GMMA_Previous that means the trend is going up
            //  so increment the GMMA_Count to reflect this.)
            //
            GMMA_Count++;
         }
         else
         {
            //
            // Check to see if GMMA_Current is less than GMMA_Previous.
            // 
            if (GMMA_Current < GMMA_Previous) 
            {
               //
               // GMMA_Current is less than GMMA_Previous so decrease the GMMA_Count by 1.
               //
               // (If GMMA_Current is less than GMMA_Previous that means the trend is going down
               //  so decrease the GMMA_Count to reflect this.)
               //
               GMMA_Count--;
            }
         }
      }
      //
      // Check to see if GMMA_Count equals our current GMMA_Array_Size.
      //
      if (GMMA_Count == GMMA_Array_Size[array_loop]) 
      {
         //
         // Since the GMMA_Counter equals the current GMMA_Array_Size set the GMMA_Trend for this color of the
         // GMMA "rainbow" and timeframe to "long".
         //
         // (Since we have have counted the same amount of GMMA lines going the same positive direction, the overall
         //  trend is "long".)
         //
         GMMA_Trend[array_loop][timeframe_index] = Long_String;
      }
      else
      {
         if (GMMA_Count == -GMMA_Array_Size[array_loop])
         {
            //
            // Since the GMMA_Counter equals the negative of the current GMMA_Array_Size set the GMMA_Trend for 
            // this color of the GMMA "rainbow" and timeframe to "short".
            //
            // (Since we have counted the same amount of GMMA lines going the same negative direction, the overall
            //  trend is "short".)
            //
            GMMA_Trend[array_loop][timeframe_index] = Short_String;
         }
         else
         {
            //
            // Anything else is a "flat" signal so set the GMMA_Trend for this color of the GMMA "rainbow" and timeframe
            // to "flat". 
            //
            GMMA_Trend[array_loop][timeframe_index] = Flat_String;
         }
      }
      //
      // Put our computed GMMAMax into our GMMA_Max array.
      //
      GMMA_Max[array_loop][timeframe_index] = GMMAMax;
      //
      // Put our computed GMMAMin into our GMMA_Min array.
      //
      GMMA_Min[array_loop][timeframe_index] = GMMAMin;
   }
   //
   // Check to see if the red and green GMMA are "long" and if all the green GMMA lines are above all the red
   // GMMA lines.
   //
   if (GMMA_Trend[4][timeframe_index]==Long_String && GMMA_Trend[3][timeframe_index]==Long_String && GMMA_Max[4][timeframe_index] < GMMA_Min[3][timeframe_index])
   {
      //
      // The red and green GMMA are "long" and all the green GMMA lines are above all the red GMMS lines so the
      // investor trend is "long".
      //
      GMMA_Trend[5][timeframe_index]=Long_String;
   }
   else
   {
      //
      // Check to see if the red and green GMMA lines are "short" and if all the red GMMA lines are above all the
      // green GMMA lines.
      //
      if (GMMA_Trend[4][timeframe_index]==Short_String && GMMA_Trend[3][timeframe_index]==Short_String && GMMA_Min[4][timeframe_index] > GMMA_Max[3][timeframe_index])
      {
         //
         // The red and green GMMA are "short" and all the red GMMA lines are above all the green GMMA lines so the
         // investor trend is "short". 
         //
         GMMA_Trend[5][timeframe_index]=Short_String;
      }
      else
      {
         //
         // The investor trend is "flat".
         //
         GMMA_Trend[5][timeframe_index]=Flat_String;
      }
   } 
   return(0);
}

//+------------------------------------------------------------------+
//| Compute_EMA_Crossover_Trend                                      |
//| Written By: Robert Alan Byer                                     |
//|             byer@mail.ourservers.net                             |
//|                                                                  |
//| Based On Original Work By Forex Factory Member: Paul Murfin      |
//|                                                                  |
//| Compute the moving average crossover values for the time periods |
//| passed to us for the "Rainbow Trend Follower" system outlined    |
//| at..                                                             |
//|                                                                  |
//| http://www.bustathief.com/earn-money-with-forex-trend-follower/  |
//| http://www.forexfactory.com/showthread.php?t=40325               |
//+------------------------------------------------------------------+
void Compute_EMA_Crossover_Trend(int timeframe_index, int timeframe_period)
{
                            
   double Fast_EMA_Current;      // Current fast EMA value.
   double Fast_EMA_Previous;     // Previous fast EMA value.
   double Slow_EMA_Current;      // Current slow EMA value.
   double Slow_EMA_Previous;     // Previous slow EMA value.

   //
   // Compute the current fast EMA for the current symbol taking the timeframe from the
   // Chart_Timeframe array with a average period of "4", a MA shift of "0" using EMA mode
   // with the closing price applied and an index of "1".
   //
   Fast_EMA_Current = iMA(NULL, timeframe_period, 4, 0, MODE_EMA, PRICE_CLOSE, 1);
   //
   // Compute the current slow EMA for the current symbol taking the timeframe from the
   // Chart_Timeframe array with the average period of "8", a MA shift of "0" using EMA mode
   // with the closing price applied and an index of "1".
   //
   Slow_EMA_Current = iMA(NULL, timeframe_period, 8, 0, MODE_EMA, PRICE_CLOSE, 1);
   //
   // Compute the previous fast EMA for the current symbol taking the timeframe from the
   // Chart_Timeframe array with an average period of "4", an MA shift of "0" using EMA mode
   // with the closing price applied and an index of "2".
   //
   Fast_EMA_Previous = iMA(NULL, timeframe_period, 4, 0, MODE_EMA, PRICE_CLOSE, 2);
   //
   // Compute the previous slow EMA for the current symbol taking the timeframe from the
   // Chart_Timeframe array with the average period of "8", a MA shift of "0" using EMA mode 
   // with the closing price applied and an index of "2".
   //
   Slow_EMA_Previous = iMA(NULL, timeframe_period, 8, 0, MODE_EMA, PRICE_CLOSE, 2);
   //
   // Check to see if the Fast_EMA_Current is greater than Slow_EMA_Current and if Fast_EMA_Previous is
   // less than Slow_EMA_Previous.
   //
   if (Fast_EMA_Current > Slow_EMA_Current && Fast_EMA_Previous < Slow_EMA_Previous)
   {
      //
      // We have a "long" EMA crossover trend, so set the EMA crossover trend for this timeframe to "long".
      //
      Trend_Array[0][timeframe_index] = Long_String;
   }
   else
   {
      //
      // Check to see if the Fast_EMA_Current is less than Slow_EMA_Current and if Fast_EMA_Previous is
      // greater than Slow_EMA_Previous.
      //
      if (Fast_EMA_Current < Slow_EMA_Current && Fast_EMA_Previous > Slow_EMA_Previous)
      {
         //
         // We have a "short" EMA crossover trend, so set the EMA crossover trend for this timeframe to "short".
         //
         Trend_Array[0][timeframe_index] = Short_String;
      }  
      else
      {
         //
         // Compute a new current fast EMA for the current symbol taking the timeframe from the
         // Chart_Timeframe array with a average period of "4", a MA shift of "0" using EMA mode
         // with the closing price applied and an index of "2".
         //
         Fast_EMA_Current = iMA(NULL, timeframe_period, 4, 0, MODE_EMA, PRICE_CLOSE, 2);
         //
         // Compute a new current slow EMA for the current symbol taking the timeframe from the
         // Chart_Timeframe array with the average period of "8", a MA shift of "0" using EMA mode
         // with the closing price applied and an index of "2".
         //
         Slow_EMA_Current = iMA(NULL, timeframe_period, 8, 0, MODE_EMA, PRICE_CLOSE, 2);
         //
         // Compute a new previous fast EMA for the current symbol taking the timeframe from the
         // Chart_Timeframe array with an average period of "4", an MA shift of "0" using EMA mode
         // with the closing price applied and an index of "4".
         //
         Fast_EMA_Previous = iMA(NULL, timeframe_period, 4, 0, MODE_EMA, PRICE_CLOSE, 3);
         //
         // Compute a new previous slow EMA for the current symbol taking the timeframe from the
         // Chart_Timeframe array with the average period of "8", a MA shift of "0" using EMA mode 
         // with the closing price applied and an index of "4".
         //
         Slow_EMA_Previous = iMA(NULL, timeframe_period, 8, 0, MODE_EMA, PRICE_CLOSE, 3);
         //
         // Check to see if the Fast_EMA_Current is greater than Slow_EMA_Current and if Fast_EMA_Previous is
         // less than Slow_EMA_Previous.
         //
         if (Fast_EMA_Current > Slow_EMA_Current && Fast_EMA_Previous < Slow_EMA_Previous)
         {
            //
            // We have a "long" EMA crossover trend, so set the EMA crossover trend for this timeframe to "long".
            //
            Trend_Array[0][timeframe_index] = Long_String;
         }
         else
         {
            //
            // Check to see if the Fast_EMA_Current is less than Slow_EMA_Current and if Fast_EMA_Previous is
            // greater than Slow_EMA_Previous.
            //
            if (Fast_EMA_Current < Slow_EMA_Current && Fast_EMA_Previous > Slow_EMA_Previous)
            {
               //
               // We have a "short" EMA crossover trend, so set the EMA crossover trend for this timeframe to "short".
               //
               Trend_Array[0][timeframe_index] = Short_String;
            }  
            else
            {
               //
               // We have a "flat" EMA crossover trend, so set the EMA crossover trend for this timeframe to "flat".
               //
               Trend_Array[0][timeframe_index] = Flat_String;
            }
         }
      }
   }
   return(0);
}

//+------------------------------------------------------------------+
//| Compute_MACD_Trend                                               |
//| Written By: Robert Alan Byer                                     |
//|             byer@mail.ourservers.net                             |
//|                                                                  |
//| Based On Original Work By Forex Factory Member: Paul Murfin      |
//|                                                                  |
//| Compute the MACD values for the time periods passed to us for    |
//| the "Rainbow Trend Follower" system outlined at...               |
//|                                                                  |
//| http://www.bustathief.com/earn-money-with-forex-trend-follower/  |
//| http://www.forexfactory.com/showthread.php?t=40325               |
//+------------------------------------------------------------------+
void Compute_MACD_Trend(int timeframe_index, int timeframe_period)
{
                               
   //
   // Compute the moving average convergence/divergence for the current chart symbol using the timeframe
   // taken from the Chart_Timeframe array with fast EMA period set to 5, slow EMA period set to 35, signal
   // period set to 5, applied price is the close price, base indicator line with a index shift of 1.
   // 
	MACD_Value[timeframe_index] = iMACD(NULL,timeframe_period,5,35,5,PRICE_CLOSE,MODE_MAIN,1);
	//
	// Check to see if the MACD_Value is greater than 0.
	//
   if (MACD_Value[timeframe_index] > 0) 
   {
     //
     // The MACD_Value is greater than 0 so set the MACD trend for this timeframe to "long".
     //
     Trend_Array[1][timeframe_index] = Long_String;
   }
   else
   {
      //
      // Check to see if the MACD_Value is less than 0.
      //
      if (MACD_Value[timeframe_index] < 0) 
      {
         //
         // The MACD_Value is less than 0 so set the MACD trend for this timeframe to "short".
         //
         Trend_Array[1][timeframe_index] = Short_String;
      }
      else
      {
         //
         // Anything else is a "flat" signal so set the MACD_Trend for this timeframe to "flat".
         //
         Trend_Array[1][timeframe_index] = Flat_String;
      }
   }
   return(0);
}

//+------------------------------------------------------------------+
//| Compute_Laguerre_Trend                                           |
//| Written By: Robert Alan Byer                                     |
//|             byer@mail.ourservers.net                             |
//|                                                                  |
//| Original Work By: Emerald King                      |
//|                   mailto:info@emerald-king.com                        |
//|                                                                  |
//| Modified for multitimeframes by:  Robert Alan Byer               |
//|                                   byer@mail.ourservers.net       |
//|                                                                  |
//| Compute the Laguerre value for the time periods passed to us for |
//| the "Rainbow Trend Follower" system outlined at...               |
//|                                                                  |
//| http://www.bustathief.com/earn-money-with-forex-trend-follower/  |
//| http://www.forexfactory.com/showthread.php?t=40325               |
//+------------------------------------------------------------------+
void Compute_Laguerre_Trend(int timeframe_index, int timeframe_period)
{

   int Bar_Counter;

   double gamma = 0.66;
   double L0 = 0;
   double L1 = 0;
   double L2 = 0;
   double L3 = 0;
   double L0A = 0;
   double L1A = 0;
   double L2A = 0;
   double L3A = 0;
   double CD = 0;
   double CU = 0;
   double LRSI[];
   
   Bar_Counter = iBars(NULL, timeframe_period) - 1;
   ArrayResize (LRSI, Bar_Counter);
   //
   // While we have bars...
   //
   while (Bar_Counter >= 0)
   {
      L0A = L0;
      L1A = L1;
      L2A = L2;
      L3A = L3;
      L0 = (1 - gamma)*iClose(NULL, timeframe_period, Bar_Counter) + gamma*L0A;
      L1 = - gamma *L0 + L0A + gamma *L1A;
      L2 = - gamma *L1 + L1A + gamma *L2A;
      L3 = - gamma *L2 + L2A + gamma *L3A;
      
      CU = 0;
      CD = 0;
      
      if (L0 >= L1) CU = L0 - L1; else CD = L1 - L0;
      if (L1 >= L2) CU = CU + L1 - L2; else CD = CD + L2 - L1;
      if (L2 >= L3) CU = CU + L2 - L3; else CD = CD + L3 - L2;

      if (CU + CD != 0) LRSI[Bar_Counter] = CU / (CU + CD);
      Bar_Counter--;
   }
   
   if (LRSI[1] > 0.15 && LRSI[2] < 0.15) 
   {
      Trend_Array[2][timeframe_index] = Long_String;
      Laguerre_Value[timeframe_index] = LRSI[1];
   }
   else
   {
     if (LRSI[1] < 0.75 && LRSI[2] > 0.75) 
     {
        Trend_Array[2][timeframe_index] = Short_String;
        Laguerre_Value[timeframe_index] = LRSI[1];
     }
     else
     {
        if (LRSI[2] > 0.15 && LRSI[3] < 0.15) 
        {
            Trend_Array[2][timeframe_index] = Long_String;
            Laguerre_Value[timeframe_index] = LRSI[2];
        }
        else
        {
           if (LRSI[2] < 0.75 && LRSI[3] > 0.75) 
           {
              Trend_Array[2][timeframe_index] = Short_String;
              Laguerre_Value[timeframe_index] = LRSI[2];
           }
           else
           {
              Trend_Array[2][timeframe_index] = Flat_String;
              Laguerre_Value[timeframe_index] = LRSI[2];
           }
        }
     }
  }           
   return(0);
}

//+------------------------------------------------------------------+
//| Compute_Overall_GMMA_Trend                                       |
//| Written By: Robert Alan Byer                                     |
//|             byer@mail.ourservers.net                             |
//|                                                                  |
//| Based On Original Work By Forex Factory Member: Paul Murfin      |
//|                                                                  |
//| Compute the overall GMMA trend for the time periods passed to us |
//| for the "Rainbow Trend Follower" system outlined at...           |
//|                                                                  |
//| http://www.bustathief.com/earn-money-with-forex-trend-follower/  |
//| http://www.forexfactory.com/showthread.php?t=40325               |
//+------------------------------------------------------------------+
void Compute_Overall_GMMA_Trend(int timeframe_index)
{

   //
   // Check to see if the yellow, orange, blue, green and red trends for the timeframe are "long" and
   // if all the green GMMA lines are above all the red GMMA lines.
   //
   if (GMMA_Trend[0][timeframe_index] == Long_String && GMMA_Trend[1][timeframe_index] == Long_String && 
       GMMA_Trend[2][timeframe_index] == Long_String && GMMA_Trend[5][timeframe_index] == Long_String)
   {
      //
      // Everything is "long", so set the overall GMMA trend for this timeframe to "long".
      //
      Trend_Array[3][timeframe_index] = Long_String;
   }
   else
   {
      //
      // Check to see if the yellow, orange, blue and investor trends for the timeframe are "short".
      //
      if (GMMA_Trend[0][timeframe_index] == Short_String && GMMA_Trend[1][timeframe_index] == Short_String && 
          GMMA_Trend[2][timeframe_index] == Short_String && GMMA_Trend[5][timeframe_index] == Short_String)
      {
         //
         // Everything is "short", so set the overall GMMA trend for this timeframe to "short".
         //
         Trend_Array[3][timeframe_index] = Short_String;
      }
      else
      {
         //
         // Everthing else is "flat" so set the overall GMMA trend for this timeframe to "flat".
         //
         Trend_Array[3][timeframe_index] = Flat_String;
      }
   }
   return(0);
}

//+------------------------------------------------------------------+
//| Compute_Trade_Suggestion                                         |
//| Written By: Robert Alan Byer                                     |
//|             byer@mail.ourservers.net                             |
//|                                                                  |
//| Based On Original Work By Forex Factory Member: Paul Murfin      |
//|                                                                  |
//| Compute the trade suggestion for the time periods passed to us   |
//| for the "Rainbow Trend Follower" system outlined at...           |
//|                                                                  |
//| http://www.bustathief.com/earn-money-with-forex-trend-follower/  |
//| http://www.forexfactory.com/showthread.php?t=40325               |
//+------------------------------------------------------------------+
void Compute_Trade_Suggestion(int timeframe_index)
{
      
   //
   // Check to see if the EMA crossover, MACD, Laguerre and over all GMMA trends are "long".
   //
   if (Trend_Array[0][timeframe_index] == Long_String && Trend_Array[1][timeframe_index] == Long_String && 
       Trend_Array[2][timeframe_index] == Long_String && Trend_Array[3][timeframe_index] == Long_String)
   {
      //
      // Everything is "long", so set the trade suggestion for this timeframe to "long".
      //
      Trend_Array[4][timeframe_index] = Long_String;
   }
   else
   {
      //
      // Check to see if the EMA crossover, MACD, Laguerre and over all GMMA trends are "short".
      //
      if (Trend_Array[0][timeframe_index] == Short_String && Trend_Array[1][timeframe_index] == Short_String && 
          Trend_Array[2][timeframe_index] == Short_String && Trend_Array[3][timeframe_index] == Short_String)
      {
         //
         // Everything is "short", so set the trade suggestion for this timeframe to "short".
         //
         Trend_Array[4][timeframe_index] = Short_String;
      }
      else
      {
         //
         // Everthing else is "flat" so set the trade suggestion for this timeframe to "flat".
         //
         Trend_Array[4][timeframe_index] = Flat_String;
      }
   }
   return(0);
}

//+------------------------------------------------------------------+
//| Chart_Period_Name                                                |
//| Written By: Robert Alan Byer                                     |
//|             byer@mail.ourservers.net                             |
//|                                                                  |
//| Figure out the name (ie. "M1, M5, M15" etc.) for the current     |
//| chart period.                                                    |
//+------------------------------------------------------------------+
string Chart_Period_Name(int chart_period)
{
   int array_loop;         // Loop variable to loop through the
                           // Chart_Timeframe array.

   //
   // Create a loop to go through the Chart_Timeframe array.
   //
   for (array_loop=0; array_loop<ArraySize(Chart_Timeframe); array_loop++)
   {
      //
      // Check to see if the chart period passed to us matches the time period 
      // in the Chart_Timeframe array.
      //
      if (Chart_Timeframe[array_loop] == chart_period)
      {
         //
         // We found our time period so return the string description of it.
         //
         return(Chart_Period_String[array_loop]);
      }
   }
   //
   // We should always find the chart period, but just in case we don't, return "error".
   //
   return("ERROR: Chart Period");
}

//+------------------------------------------------------------------+
//| Check_NewBar                                                     |
//| Written By: Robert Alan Byer                                     |
//|             byer@mail.ourservers.net                             |
//|                                                                  |
//| Based On Original Work By Forex Factory Member: Paul Murfin      |
//|                                                                  |
//| Check to see if we have a new bar for the chart timeframe passed |
//| to us.                                                           |
//+------------------------------------------------------------------+
bool Check_NewBar(int timeframe_index, int timeframe_period)
{

   //
   // Check to see if the current chart bar time equals the last chart bar time.
   // (i.e. if we have a new chart bar).
   //
   if (iTime(NULL, timeframe_period, 0) != Last_BarTime[timeframe_index])
   {
      //
      // The current chart bar time does not equal the last chart bar time which
      // means we have a new chart bar.  Save the current chart bar time in the
      // Last_BarTime for this chart period.
      //
      Last_BarTime[timeframe_index] = iTime(NULL, timeframe_period, 0);
      //
      // Since we have a new chart bar, return a "true".
      //
      return(true);
    }
    else
    {
      //
      // The current chart bar time equals the last chart bar time which means we don't have
      // a new chart bar so return a "false".
      //
      return(false);
    }
}

//+------------------------------------------------------------------+
//| Send_EMail                                                       |
//| Written By: Robert Alan Byer                                     |
//|             byer@mail.ourservers.net                             |
//|                                                                  |
//|                                                                  |
//| Send an e-mail alert.                                            |
//+------------------------------------------------------------------+
void Send_Email(int timeframe_index, int timeframe_period)
{

   string email_subject;          // The e-mail subject line.
   string email_body;             // The e-mail body.
   int num_rows;                  // Number of rows in an array.
   
   //
   // Put the chart pair and timeframe in the subject line of the e-mail.
   //
   email_subject = "Rainbow Trend Follower Alert: " + Symbol() + " " + Chart_Period_Name(timeframe_period);
   //
   // Put the current chart bar time and chart pair and chart timeframe in the body of the e-mail.
   // (and a blank line)
   //
   email_body = TimeToStr(iTime(Symbol(), timeframe_period,0)) + ": " + Symbol() + " " + Chart_Period_Name(timeframe_period) + "\n";
   //
   // Put the current chart bar open & high.
   //
   email_body = email_body + "Open: " + iOpen(Symbol(), timeframe_period,0) + "     High: " + 
                iHigh(Symbol(), timeframe_period,0) + "\n";
   //
   // Put the current chart bar low and close.
   //
   email_body = email_body + "Low: " + iLow(Symbol(), timeframe_period,0) + "     Close: " + 
                iClose(Symbol(), timeframe_period,0 ) + "\n";
   //
   // Put the current chart bar volume.
   //
   email_body = email_body + "Volume: " + iVolume(Symbol(), timeframe_period,0) + "\n\n";
   //
   // Create a loop to go through the GMMA_Row_String and GMMA_Trend arrays.
   //
   for(num_rows=0; num_rows<ArraySize(GMMA_Row_String); num_rows++)
   {
      //
      // Add the GMMA's for the timeframe passed to us to the e-mail.
      //
      email_body = email_body + GMMA_Row_String[num_rows] + " is " + GMMA_Trend[num_rows][timeframe_index] + "\r";
   }
   //
   // Check to see if the green GMMA is above the red GMMA.
   //
   if (GMMA_Trend[5][timeframe_index]==Long_String)
   {
      //
      // The green GMMA is above the red GMMA.
      //
      email_body = email_body + "The green GMMA is above the red GMMA.\n";
   }
   //
   // Check to see if the red GMMA is above the green GMMA.
   //
   if (GMMA_Trend[5][timeframe_index]==Short_String)
   {
      //
      // The red GMMA is above the green GMMA.
      //
      email_body = email_body + "The red GMMA is above the green GMMA.\n";
   }
   //
   // Add the EMA Crossover data.
   //
   email_body = email_body + "\n" + Trend_Row_String[0] + " is " + Trend_Array[0][timeframe_index] + "\n";
   //
   // Add the MACD data. 
   //
   email_body = email_body + Trend_Row_String[1] + " is " + Trend_Array[1][timeframe_index] + " (" + MACD_Value[timeframe_index] + ")\n";
   //
   // Add the Laguerre data.
   //
   email_body = email_body + Trend_Row_String[2] + " is " + Trend_Array[2][timeframe_index] + " (" + Laguerre_Value[timeframe_index] + ")\n";
   //
   // Add the GUPPY data.
   //
   email_body = email_body + Trend_Row_String[3] + " is " + Trend_Array[3][timeframe_index] + "\n\n";
   //
   // Check to see if the trade suggestion is "long".
   //
   if (Trend_Array[4][timeframe_index] == Long_String)
   {
      //
      // The trade suggestion is "long" so tell the user to go "long".
      //
      email_body = email_body + "Trade Suggestion: Go Long, Buy!!!";
   }
   //
   // Check to see if the trade suggestion is "short".
   //
   if (Trend_Array[4][timeframe_index] == Short_String)
   {
      //
      // The trade suggestion is "short" so tell the user to to "short".
      //
      email_body = email_body + "Trade Suggestion: Go Short, Sell!!!";
   }
   //
   // Send the e-mail alert.
   //
   SendMail(email_subject, email_body);

   return(0);
}

